' SocketWrench 10.0
' Copyright 2023 Catalyst Development Corporation
' All rights reserved
'
' This file is licensed to you pursuant to the terms of the
' product license agreement included with the original software,
' and is protected by copyright law and international treaties.
' Unauthorized reproduction or distribution may result in severe
' criminal penalties.
'
Option Strict Off
Option Explicit On

Imports SocketTools.InternetServer

Public Class Server

    Private WithEvents SecureServer As New SocketTools.InternetServer
    Private ClientTable As New Hashtable

    ' The following code demonstrates how to create a method to safely
    ' update the ListBox control from another thread. Delcare a
    ' delegate that will be used with the Invoke method.

    Delegate Sub ListBoxCallback(ByVal strMessage As String)

    ' Create a public method that will update the ListBox control
    ' in the UI thread.

    Public Sub UpdateListBox(ByVal strMessage As String)
        If Me.ListBox1.InvokeRequired Then
            Dim dMethod As New ListBoxCallback(AddressOf UpdateListBox)
            Me.Invoke(dMethod, New Object() {strMessage})
        Else
            Me.ListBox1.Items.Add(strMessage)
            Me.ListBox1.SelectedIndex = Me.ListBox1.Items.Count - 1
            Me.ListBox1.ClearSelected()
        End If
    End Sub

    Public Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Initialize the InternetServer class
        If Not SecureServer.Initialize() Then
            MsgBox("Unable to initialize SocketWrench server", MsgBoxStyle.Exclamation)
            End
        End If
    End Sub

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        If SecureServer.Status = ServerStatus.serverInactive Then
            Dim strLocalAddress As String
            Dim nLocalPort As Integer
            Dim nMaxClients As Integer

            strLocalAddress = TextBox1.Text.Trim()
            nLocalPort = Integer.Parse(TextBox2.Text)
            nMaxClients = Integer.Parse(TextBox3.Text)

            If nLocalPort < 1 Or nLocalPort > 65535 Then
                MsgBox("Please specify a valid port number for the server", MsgBoxStyle.Exclamation)
                TextBox2.Focus()
                Exit Sub
            End If

            Button1.Enabled = False

            ' This specifies the name of the certificate to use with secure connections
            ' For more information on how to create a test certificate, reference
            ' https://sockettools.com/kb/creating-certificate-using-openssl/

            SecureServer.CertificateName = "localhost"
            SecureServer.Secure = True

            If SecureServer.Start(strLocalAddress, nLocalPort, nMaxClients) Then
                UpdateListBox("Server started listening for connections")
            Else
                MsgBox(SecureServer.LastErrorString, MsgBoxStyle.Exclamation)
                Button1.Enabled = True
                Exit Sub
            End If

            Button1.Text = "Stop"
            Button1.Enabled = True
        Else
            If SecureServer.Stop() Then
                UpdateListBox("Server stopped listening for connections")
            Else
                MsgBox(SecureServer.LastErrorString, MsgBoxStyle.Exclamation)
                Exit Sub
            End If

            Button1.Text = "Start"
        End If
    End Sub

    Private Sub SecureServer_OnAccept(ByVal sender As Object, ByVal e As SocketTools.InternetServer.AcceptEventArgs) Handles SecureServer.OnAccept
        UpdateListBox("Incoming client connection from " + e.ClientAddress + ":" + e.ClientPort.ToString())
    End Sub

    Private Sub SecureServer_OnConnect(ByVal sender As Object, ByVal e As SocketTools.InternetServer.ConnectEventArgs) Handles SecureServer.OnConnect
        ' Assign a unique name to the client session based on its client ID
        SecureServer.ClientName = "Client" + SecureServer.ClientId.ToString()

        ' Initialize an entry in the client hashtable
        SyncLock ClientTable.SyncRoot
            ClientTable.Add(SecureServer.ClientId, 0)
        End SyncLock

        UpdateListBox(SecureServer.ClientName + " connected to server")
    End Sub

    Private Sub SecureServer_OnDisconnect(ByVal sender As Object, ByVal e As SocketTools.InternetServer.DisconnectEventArgs) Handles SecureServer.OnDisconnect
        Dim nBytesTotal As Integer

        ' Get the total number of bytes that have been exchanged
        SyncLock ClientTable.SyncRoot
            nBytesTotal = ClientTable(SecureServer.ClientId)
        End SyncLock

        UpdateListBox(SecureServer.ClientName + " disconnected from server (" + nBytesTotal.ToString() + " bytes exchanged)")
    End Sub

    Private Sub SecureServer_OnError(ByVal sender As Object, ByVal e As SocketTools.InternetServer.ErrorEventArgs) Handles SecureServer.OnError
        UpdateListBox("Error " + e.Error.ToString() + " " + e.Description + " [" + e.Handle.ToString() + "]")
    End Sub

    Private Sub SecureServer_OnRead(ByVal sender As Object, ByVal e As SocketTools.InternetServer.ReadEventArgs) Handles SecureServer.OnRead
        Dim ioBuffer(4096) As Byte
        Dim nBytesRead As Integer
        Dim nBytesWritten As Integer
        Dim nBytesTotal As Integer

        ' Read some data that the client has sent to the server
        nBytesRead = SecureServer.Read(ioBuffer)

        If nBytesRead > 0 Then
            ' Echo the data back to the client
            nBytesWritten = SecureServer.Write(ioBuffer, nBytesRead)

            ' Update the client hashtable with the amount of data written
            SyncLock ClientTable.SyncRoot
                nBytesTotal = ClientTable(SecureServer.ClientId) + nBytesWritten
                ClientTable(SecureServer.ClientId) = nBytesTotal
            End SyncLock
        End If
    End Sub
End Class
