// SocketTools 10 Build 1468
// Copyright 2023 Catalyst Development Corporation
// All rights reserved
// 
// This file is licensed to you pursuant to the terms of the
// product license agreement included with the original software,
// and is protected by copyright law and international treaties.
// Unauthorized reproduction or distribution may result in severe
// criminal penalties.
// 
using System;
using System.Collections;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;

namespace HttpServer
{
    internal partial class formMain : System.Windows.Forms.Form 
    { 
        private SocketTools.HttpServer httpServer = new SocketTools.HttpServer();

        public delegate void UpdateFormCallback(bool bEnable);

        private void UpdateForm(bool bEnable)
        {
            if (cmdStart.InvokeRequired)
            {
                UpdateFormCallback d = new UpdateFormCallback(UpdateForm);
                this.Invoke(d, new object[] { bEnable });
            }
            else
            {
                if (httpServer.IsActive)
                {
                    cmdStart.Enabled = bEnable;
                    cmdSuspend.Enabled = bEnable;
                    cmdRestart.Enabled = bEnable;
                }
                else
                {
                    cmdStart.Enabled = bEnable;
                    cmdSuspend.Enabled = false;
                    cmdRestart.Enabled = false;
                }
            }
        }

        public delegate void UpdateStatusCallback(string strMessage);

        private void UpdateStatus(string strMessage)
        {
            if (lblStatus.InvokeRequired)
            {
                UpdateStatusCallback d = new UpdateStatusCallback(UpdateStatus);
                this.Invoke(d, new object[] { strMessage });
            }
            else
            {
                lblStatus.Text = strMessage;
                lblClient.Text = httpServer.ClientCount + " Clients";
            }
        }

        private void formMain_Load(System.Object eventSender, System.EventArgs eventArgs)
        {
            comboFormats.Items.Insert(0, "None");
            comboFormats.Items.Insert(1, "Common");
            comboFormats.Items.Insert(2, "Extended");
            comboFormats.SelectedIndex = 0;
        }

        private void cmdStart_Click(System.Object eventSender, System.EventArgs eventArgs)
        {
            if (!(httpServer.IsActive))
            {
                UpdateForm(false);

                //  Basic server configuration
                httpServer.ServerName = editHostName.Text.Trim();
                httpServer.ServerAddress = editLocalAddress.Text.Trim();
                httpServer.ServerPort = Int32.Parse(editLocalPort.Text);
                httpServer.Identity = editIdentity.Text.Trim();
                httpServer.Directory = editDirectory.Text.Trim();

                //  Server limits
                httpServer.MaxClients = Int32.Parse(editMaxClients.Text);
                httpServer.IdleTime = Int32.Parse(editIdleTime.Text);

                //  Server options
                httpServer.Secure = Convert.ToBoolean(chkSecure.CheckState);
                httpServer.LocalUser = Convert.ToBoolean(chkLocalUser.CheckState);
                httpServer.MultiUser = Convert.ToBoolean(chkMultiUser.CheckState);
                httpServer.LockFiles = Convert.ToBoolean(chkLockFiles.CheckState);
                httpServer.Restricted = Convert.ToBoolean(chkRestricted.CheckState);
                httpServer.ReadOnly = Convert.ToBoolean(chkReadOnly.CheckState);
                httpServer.HiddenFiles = Convert.ToBoolean(chkHidden.CheckState);

                //  Server logging
                httpServer.LogFormat = ((SocketTools.HttpServer.FormatType)(comboFormats.SelectedIndex));
                httpServer.LogFile = editLogFile.Text.Trim();

                //  Start the server
                if (httpServer.Start())
                {
                    cmdStart.Text = "Stop";
                    UpdateStatus("Server accepting connections on " + httpServer.ServerAddress + ":" + httpServer.ServerPort);
                    UpdateForm(true);
                }
                else
                {
                    MessageBox.Show(httpServer.LastErrorString);
                    UpdateForm(true);
                    return;
                }
            }
            else
            {
                UpdateForm(false);

                if (httpServer.Stop())
                {
                    cmdStart.Text = "Start";
                }
                else
                {
                    MessageBox.Show(httpServer.LastErrorString);
                    UpdateForm(true);
                    return;
                }
            }
        }

        private void cmdSuspend_Click(System.Object eventSender, System.EventArgs eventArgs)
        {
            if (cmdSuspend.Text == "Suspend")
            {
                UpdateForm(false);

                if (httpServer.Suspend())
                {
                    cmdSuspend.Text = "Resume";
                    UpdateStatus("Server suspended accepting new client connections");
                    UpdateForm(true);
                }
                else
                {
                    MessageBox.Show(httpServer.LastErrorString);
                    UpdateForm(true);
                    return;
                }
            }
            else
            {
                UpdateForm(false);

                if (httpServer.Resume())
                {
                    cmdSuspend.Text = "Suspend";
                    UpdateStatus("Server resumed accepting new client connections");
                    UpdateForm(true);
                }
                else
                {
                    MessageBox.Show(httpServer.LastErrorString);
                    UpdateForm(true);
                    return;
                }
            }
        }

        private void cmdRestart_Click(System.Object eventSender, System.EventArgs eventArgs)
        {
            UpdateForm(false);

            if (httpServer.Restart())
            {
                cmdSuspend.Text = "Suspend";
                UpdateStatus("The server has been restarted");
                UpdateForm(true);
            }
            else
            {
                MessageBox.Show(httpServer.LastErrorString);
                UpdateForm(true);
                return;
            }
        }

        private void cmdClose_Click(System.Object eventSender, System.EventArgs eventArgs)
        {
            if (httpServer.IsActive)
            {
                UpdateForm(false);

                if (httpServer.Stop())
                {
                    UpdateForm(true);
                }
                else
                {
                    MessageBox.Show(httpServer.LastErrorString);
                    UpdateForm(true);
                    return;
                }
            }

            this.Close();
        }

        private void httpServer_OnAuthenticate(System.Object eventSender, SocketTools.HttpServer.AuthenticateEventArgs eventArgs)
        {
            Debug.WriteLine("Client " + eventArgs.ClientId + " authenticating as " + eventArgs.UserName);
        }

        private void httpServer_OnCommand(System.Object eventSender, SocketTools.HttpServer.CommandEventArgs eventArgs)
        {
            Debug.WriteLine("Client " + eventArgs.ClientId + " sent command " + eventArgs.Command + " [" + eventArgs.Parameters + "]");
            Debug.WriteLine("Virtual Path: " + httpServer.VirtualPath);
            Debug.WriteLine("Local Path:   " + httpServer.LocalPath);
        }

        private void httpServer_OnConnect(System.Object eventSender, SocketTools.HttpServer.ConnectEventArgs eventArgs)
        {
            Debug.WriteLine("Client " + eventArgs.ClientId + " connected from " + eventArgs.ClientAddress);
            UpdateStatus("Client " + eventArgs.ClientId + " connected from " + eventArgs.ClientAddress);
        }

        private void httpServer_OnDisconnect(System.Object eventSender, SocketTools.HttpServer.DisconnectEventArgs eventArgs)
        {
            Debug.WriteLine("Client " + eventArgs.ClientId + " disconnected");
            UpdateStatus("Client " + eventArgs.ClientId + " disconnected from the server");
        }

        private void httpServer_OnDownload(System.Object eventSender, SocketTools.HttpServer.DownloadEventArgs eventArgs)
        {
            Debug.WriteLine("Client " + eventArgs.ClientId + " downloaded " + eventArgs.FileName + " (" + eventArgs.FileSize + " bytes)");
        }

        private void httpServer_OnError(System.Object eventSender, SocketTools.HttpServer.ErrorEventArgs eventArgs)
        {
            Debug.WriteLine("Client " + eventArgs.ClientId + " error " + eventArgs.Error + ": " + eventArgs.Description);
        }

        private void httpServer_OnExecute(System.Object eventSender, SocketTools.HttpServer.ExecuteEventArgs eventArgs)
        {
            Debug.WriteLine("Client " + eventArgs.ClientId + " executed " + eventArgs.Resource + " (" + eventArgs.ExitCode + ")");
        }

        private void httpServer_OnIdle(System.Object eventSender, System.EventArgs eventArgs)
        {
            Debug.WriteLine("All clients have disconnected, the server is idle");
        }

        private void httpServer_OnResult(System.Object eventSender, SocketTools.HttpServer.ResultEventArgs eventArgs)
        {
            Debug.WriteLine("Client " + eventArgs.ClientId + " command " + eventArgs.Command + " completed (" + eventArgs.ResultCode + ")");
        }

        private void httpServer_OnStart(System.Object eventSender, System.EventArgs eventArgs)
        {
            Debug.WriteLine("Server has started accepting connections");
            UpdateStatus("Server accepting connections on " + httpServer.ServerAddress + ":" + httpServer.ServerPort);
            UpdateForm(true);
        }

        private void httpServer_OnStop(System.Object eventSender, System.EventArgs eventArgs)
        {
            Debug.WriteLine("Server has stopped, all clients terminated");
            UpdateStatus("Server stopped accepting connections");
            UpdateForm(true);
        }

        private void httpServer_OnTimeout(System.Object eventSender, SocketTools.HttpServer.TimeoutEventArgs eventArgs)
        {
            Debug.WriteLine("Client " + eventArgs.ClientId + " timed out after " + eventArgs.Elapsed + " seconds");
        }

        private void httpServer_OnUpload(System.Object eventSender, SocketTools.HttpServer.UploadEventArgs eventArgs)
        {
            Debug.WriteLine("Client " + eventArgs.ClientId + " uploaded " + eventArgs.FileName + " (" + eventArgs.FileSize + " bytes)");
        }
    }
} 
