// SocketTools 10 Build 1468
// Copyright 2023 Catalyst Development Corporation
// All rights reserved
// 
// This file is licensed to you pursuant to the terms of the
// product license agreement included with the original software,
// and is protected by copyright law and international treaties.
// Unauthorized reproduction or distribution may result in severe
// criminal penalties.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.ServiceProcess;
using System.Text;
using System.Threading;
using SocketTools;

namespace EchoService
{
    public partial class Service1 : ServiceBase
    {
        // Service constants
        private const int servicePort = 7000;
        private const int serviceInterval = 5;
        private const int serviceTimeout = 60;
        private const int bufferSize = 1024;

        private Thread m_serverThread = null;
        private static ManualResetEvent m_threadEvent = new ManualResetEvent(false);
        private SocketTools.SocketWrench m_serverSocket = new SocketTools.SocketWrench();

        public Service1()
        {
            InitializeComponent();
        }

        protected override void OnStart(string[] args)
        {
            m_serverThread = new Thread(new ThreadStart(ServerMain));
            m_serverThread.Start();

            EventLog.WriteEntry(this.ServiceName, "Service started successfully.", EventLogEntryType.Information);
        }

        protected override void OnStop()
        {
            this.RequestAdditionalTime(4000);

            // Signal the worker thread to exit.
            if ((m_serverThread != null) && (m_serverThread.IsAlive))
            {
                m_threadEvent.Reset();
                Thread.Sleep(5000);
                m_serverThread.Abort();
            }

            EventLog.WriteEntry(this.ServiceName, "Service stopped successfully.", EventLogEntryType.Information);
            this.ExitCode = 0;
        }

        public void ServerMain()
        {
            int nThreadId = Thread.CurrentThread.ManagedThreadId;

            if (!m_serverSocket.Initialize())
            {
                EventLog.WriteEntry(this.ServiceName, m_serverSocket.LastErrorString, EventLogEntryType.Warning);
                return;
            }

            EventLog.WriteEntry(this.ServiceName, "Starting server thread " + nThreadId.ToString());

            // Create an instance of the SocketWrench class and begin
            // listening for client connections
            m_serverSocket.Listen(servicePort);

            try
            {
                do
                {
                    SocketTools.SocketWrench clientSocket = null;

                    if (clientSocket == null)
                    {
                        clientSocket = new SocketTools.SocketWrench();
                        clientSocket.Timeout = serviceInterval;

                        if (! clientSocket.Initialize())
                        {
                            EventLog.WriteEntry(this.ServiceName, "Error on server thread " + nThreadId.ToString() + ": " + m_serverSocket.LastErrorString, EventLogEntryType.Warning);
                            return;
                        }
                    }

                    // Check for any incoming client connections; as soon as a client
                    // connects with the server, this method will return immediately
                    if (clientSocket.Accept(m_serverSocket.Handle))
                    {
                        Thread clientThread = new Thread(new ParameterizedThreadStart(ClientSession));
                        clientThread.Start(clientSocket);
                        clientSocket = null;
                    }
                    else
                    {
                        if (clientSocket.LastError != SocketWrench.ErrorCode.errorOperationTimeout)
                        {
                            EventLog.WriteEntry(this.ServiceName, "Error on server thread " + nThreadId.ToString() + ": " + m_serverSocket.LastErrorString, EventLogEntryType.Warning);
                        }
                    }
                }
                while (true);
            }
            catch (ThreadAbortException)
            {
                EventLog.WriteEntry(this.ServiceName, "Server thread " + nThreadId.ToString() + " aborted");
            }

            EventLog.WriteEntry(this.ServiceName, "Server thread " + nThreadId.ToString() + " terminating");
        }

        public void ClientSession(object data)
        {
            SocketTools.SocketWrench clientSocket = (SocketTools.SocketWrench)data;
            int nThreadId = Thread.CurrentThread.ManagedThreadId;
            byte[] byBuffer = new byte[bufferSize];
            int nBytesRead = 0;
            int nBytesWritten = 0;
            int nBytesTotal = 0;

            clientSocket.AttachThread();
            clientSocket.Timeout = serviceTimeout;

            EventLog.WriteEntry(this.ServiceName, "Client thread " + nThreadId.ToString() + " accepted connection from " + clientSocket.PeerAddress);

            // Begin reading data from the client and sending whatever
            // is read back to them; if an error occurs, then it is logged
            while (true)
            {
                nBytesRead = clientSocket.Read(byBuffer);
                if (nBytesRead > 0)
                {
                    nBytesWritten = clientSocket.Write(byBuffer, nBytesRead);
                    if (nBytesWritten == -1)
                    {
                        EventLog.WriteEntry(this.ServiceName, "Error on client thread " + nThreadId.ToString() + ": " + clientSocket.LastErrorString, EventLogEntryType.Warning);
                        break;
                    }
                    nBytesTotal += nBytesRead;
                }
                else
                {
                    // A return value of 0 from the Read method indicates that the
                    // client has terminated the connection with the server gracefully;
                    // a return value of -1 indicates an error occurred
                    if (nBytesRead == -1)
                    {
                        EventLog.WriteEntry(this.ServiceName, "Error on client thread " + nThreadId.ToString() + ": " + clientSocket.LastErrorString, EventLogEntryType.Warning);
                    }
                    break;
                }
            }

            clientSocket.Disconnect();
            EventLog.WriteEntry(this.ServiceName, "Client thread " + nThreadId.ToString() + " terminating, " + nBytesTotal.ToString() + " bytes of data echoed");
        }
    }
}
