// SocketTools 10 Build 1468
// Copyright 2023 Catalyst Development Corporation
// All rights reserved
//
// This file is licensed to you pursuant to the terms of the
// product license agreement included with the original software,
// and is protected by copyright law and international treaties.
// Unauthorized reproduction or distribution may result in severe
// criminal penalties.
//

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using SocketTools;

namespace EchoServer
{
    public partial class MainForm : Form
    {
        private InternetServer echoServer = new InternetServer();
        private Hashtable clientTable = new Hashtable();

        // The following code demonstrates how to create a method to safely
        // update the ListBox control from another thread. Delcare a
        // delegate that will be used with the Invoke method.

        public delegate void ListBoxCallback(string strMessage);

        // Create a public method that will update the ListBox control
        // in the UI thread.

        public void UpdateListBox(string strMessage)
        {
            if (this.listBox1.InvokeRequired)
            {
                ListBoxCallback dMethod = new ListBoxCallback(UpdateListBox);
                this.Invoke(dMethod, new object[] { strMessage });
            }
            else
            {
                listBox1.Items.Add(strMessage);
                listBox1.SelectedIndex = listBox1.Items.Count - 1;
                listBox1.ClearSelected();
            }
        }

        public MainForm()
        {
            InitializeComponent();

            if (!echoServer.Initialize())
            {
                MessageBox.Show("Unable to initialize SocketWrench server");
                Application.Exit();
            }

            echoServer.OnAccept += new InternetServer.OnAcceptEventHandler(echoServer_OnAccept);
            echoServer.OnConnect += new InternetServer.OnConnectEventHandler(echoServer_OnConnect);
            echoServer.OnDisconnect += new InternetServer.OnDisconnectEventHandler(echoServer_OnDisconnect);
            echoServer.OnError += new InternetServer.OnErrorEventHandler(echoServer_OnError);
            echoServer.OnRead += new InternetServer.OnReadEventHandler(echoServer_OnRead);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (echoServer.Status == InternetServer.ServerStatus.serverInactive)
            {
                string strLocalAddress = textBox1.Text.Trim();
                int nLocalPort = int.Parse(textBox2.Text);
                int nMaxClients = int.Parse(textBox3.Text);

                if (nLocalPort < 0 || nLocalPort > 65535)
                {
                    MessageBox.Show("Please specify a valid port number for the server");
                    textBox2.Focus();
                    return;
                }

                button1.Enabled = false;

                if (echoServer.Start(strLocalAddress, nLocalPort, nMaxClients))
                {
                    UpdateListBox("Server has started listening for connections");
                }
                else
                {
                    MessageBox.Show(echoServer.LastErrorString);
                    button1.Enabled = true;
                    return;
                }

                button1.Text = "Stop";
                button1.Enabled = true;
            }
            else
            {
                if (echoServer.Stop())
                {
                    UpdateListBox("Server has stopped listening for connections");
                }
                else
                {
                    MessageBox.Show(echoServer.LastErrorString);
                    return;
                }

                button1.Text = "Start";
            }
        }

        void echoServer_OnAccept(object sender, InternetServer.AcceptEventArgs e)
        {
            UpdateListBox("Incoming client connection from " + e.ClientAddress + ":" + e.ClientPort.ToString());
        }

        void echoServer_OnConnect(object sender, InternetServer.ConnectEventArgs e)
        {
            // Assign a unique name to the client session based on its client ID
            echoServer.ClientName = "Client" + echoServer.ClientId.ToString();

            // Initialize an entry in the client hashtable
            lock(clientTable.SyncRoot)
            {
                clientTable.Add((object)echoServer.ClientId, (object)0);
            }

            UpdateListBox(echoServer.ClientName + " connected to server");
        }

        void echoServer_OnDisconnect(object sender, InternetServer.DisconnectEventArgs e)
        {
            int nBytesTotal;

            // Get the total number of bytes that have been exchanged
            lock(clientTable.SyncRoot)
            {
                nBytesTotal = (int)clientTable[(object)echoServer.ClientId];
            }

            UpdateListBox(echoServer.ClientName + " disconnected from server (" + nBytesTotal.ToString() + " bytes exchanged)");
        }

        void echoServer_OnError(object sender, InternetServer.ErrorEventArgs e)
        {
            UpdateListBox("Error " + e.Error.ToString() + " " + e.Description + " [" + e.Handle.ToString() + "]");
        }

        void echoServer_OnRead(object sender, InternetServer.ReadEventArgs e)
        {
            byte[] ioBuffer = new byte[4096];
            int nBytesRead;
            int nBytesWritten;
            int nBytesTotal;

            // Read some data that the client has sent to the server
            nBytesRead = echoServer.Read(ioBuffer);

            if (nBytesRead > 0)
            {
                // Echo the data back to the client
                nBytesWritten = echoServer.Write(ioBuffer, nBytesRead);

                // Update the client hashtable with the amount of data written
                lock (clientTable.SyncRoot)
                {
                    nBytesTotal = (int)clientTable[(object)echoServer.ClientId] + nBytesWritten;
                    clientTable[(object)echoServer.ClientId] = (object)nBytesTotal;
                }
            }
        }
    }
}