// SocketTools 10 Build 1468
// Copyright 2023 Catalyst Development Corporation
// All rights reserved
// 
// This file is licensed to you pursuant to the terms of the
// product license agreement included with the original software,
// and is protected by copyright law and international treaties.
// Unauthorized reproduction or distribution may result in severe
// criminal penalties.
// 
using System;
using System.Collections;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;

namespace FtpServer
{
    internal partial class formMain : System.Windows.Forms.Form 
    {
        private SocketTools.FtpServer ftpServer = new SocketTools.FtpServer();

        public delegate void UpdateFormCallback(bool bEnable);

        private void UpdateForm(bool bEnable)
        {
            if (cmdStart.InvokeRequired)
            {
                UpdateFormCallback d = new UpdateFormCallback(UpdateForm);
                this.Invoke(d, new object[] { bEnable });
            }
            else
            {
                if (ftpServer.IsActive)
                {
                    cmdStart.Enabled = bEnable;
                    cmdSuspend.Enabled = bEnable;
                    cmdRestart.Enabled = bEnable;
                }
                else
                {
                    cmdStart.Enabled = bEnable;
                    cmdSuspend.Enabled = false;
                    cmdRestart.Enabled = false;
                }
            }
        }

        public delegate void UpdateStatusCallback(string strMessage);

        private void UpdateStatus(string strMessage)
        {
            if (lblStatus.InvokeRequired)
            {
                UpdateStatusCallback d = new UpdateStatusCallback(UpdateStatus);
                this.Invoke(d, new object[] { strMessage });
            }
            else
            {
                lblStatus.Text = strMessage;
                lblClient.Text = ftpServer.ClientCount + " Clients";
            }
        }

        private void formMain_Load(System.Object eventSender, System.EventArgs e)
        {
            comboFormats.Items.Insert(0, "None");
            comboFormats.Items.Insert(1, "Common");
            comboFormats.Items.Insert(2, "Extended");
            comboFormats.SelectedIndex = 0;
        }

        private void cmdStart_Click(System.Object eventSender, System.EventArgs e)
        {
            int nError = 0;

            if (!ftpServer.IsActive)
            {
                UpdateForm(false);

                //  Basic server configuration
                ftpServer.ServerName = editHostName.Text.Trim();
                ftpServer.ServerAddress = editLocalAddress.Text.Trim();
                ftpServer.ServerPort = Int32.Parse(editLocalPort.Text);
                ftpServer.Identity = editIdentity.Text.Trim();
                ftpServer.Directory = editDirectory.Text.Trim();

                //  Server limits
                ftpServer.MaxClients = Int32.Parse(editMaxClients.Text);
                ftpServer.MaxGuests = Int32.Parse(editMaxGuests.Text);
                ftpServer.IdleTime = Int32.Parse(editIdleTime.Text);
                ftpServer.AuthFail = Int32.Parse(editMaxLogins.Text);
                ftpServer.MinPort = Int32.Parse(editMinPort.Text);
                ftpServer.MaxPort = Int32.Parse(editMaxPort.Text);

                //  Server options
                ftpServer.Secure = System.Convert.ToBoolean(chkSecure.CheckState);
                ftpServer.LocalUser = System.Convert.ToBoolean(chkLocalUser.CheckState);
                ftpServer.MultiUser = System.Convert.ToBoolean(chkMultiUser.CheckState);
                ftpServer.UnixMode = System.Convert.ToBoolean(chkUnixMode.CheckState);
                ftpServer.LocalTime = System.Convert.ToBoolean(chkLocalTime.CheckState);
                ftpServer.LockFiles = System.Convert.ToBoolean(chkLockFiles.CheckState);
                ftpServer.Restricted = System.Convert.ToBoolean(chkRestricted.CheckState);
                ftpServer.ReadOnly = System.Convert.ToBoolean(chkReadOnly.CheckState);
                ftpServer.HiddenFiles = System.Convert.ToBoolean(chkHidden.CheckState);

                //  Server logging
                ftpServer.LogFormat = ((SocketTools.FtpServer.FormatType)(comboFormats.SelectedIndex));
                ftpServer.LogFile = editLogFile.Text.Trim();

                //  Start the server
                if (ftpServer.Start())
                {
                    cmdStart.Text = "Stop";
                    UpdateStatus("Server accepting connections on " + ftpServer.ServerAddress + ":" + ftpServer.ServerPort);
                    UpdateForm(true);
                }
                else
                {
                    MessageBox.Show(ftpServer.LastErrorString);
                    UpdateForm(true);
                    return;
                }
            }
            else
            {
                UpdateForm(false);

                if (ftpServer.Stop())
                {
                    cmdStart.Text = "Start";
                }
                else
                {
                    MessageBox.Show(ftpServer.LastErrorString);
                    UpdateForm(true);
                    return;
                }
            }
        }

        private void cmdSuspend_Click(System.Object eventSender, System.EventArgs e)
        {
            if (cmdSuspend.Text == "Suspend")
            {
                UpdateForm(false);

                if (ftpServer.Suspend())
                {
                    cmdSuspend.Text = "Resume";
                    UpdateStatus("Server suspended accepting new client connections");
                    UpdateForm(true);
                }
                else
                {
                    MessageBox.Show(ftpServer.LastErrorString);
                    UpdateForm(true);
                    return;
                }
            }
            else
            {
                UpdateForm(false);

                if (ftpServer.Resume())
                {
                    cmdSuspend.Text = "Suspend";
                    UpdateStatus("Server resumed accepting new client connections");
                    UpdateForm(true);
                }
                else
                {
                    MessageBox.Show(ftpServer.LastErrorString);
                    UpdateForm(true);
                    return;
                }
            }
        }

        private void cmdRestart_Click(System.Object eventSender, System.EventArgs e)
        {
            UpdateForm(false);

            if (ftpServer.Restart())
            {
                cmdSuspend.Text = "Suspend";
                UpdateStatus("The server has been restarted");
                UpdateForm(true);
            }
            else
            {
                MessageBox.Show(ftpServer.LastErrorString);
                UpdateForm(true);
                return;
            }
        }

        private void cmdClose_Click(System.Object eventSender, System.EventArgs e)
        {
            if (ftpServer.IsActive)
            {
                UpdateForm(false);

                if (ftpServer.Stop())
                {
                    UpdateForm(true);
                }
                else
                {
                    MessageBox.Show(ftpServer.LastErrorString);
                    UpdateForm(true);
                    return;
                }
            }

            this.Close();
        }

        private void ftpServer_OnAuthenticate(object sender, SocketTools.FtpServer.AuthenticateEventArgs e)
        {
            Debug.WriteLine("Client " + e.ClientId + " authenticating as " + e.UserName);
        }

        private void ftpServer_OnCommand(System.Object eventSender, SocketTools.FtpServer.CommandEventArgs e)
        {
            Debug.WriteLine("Client " + e.ClientId + " sent command " + e.Command + " [" + e.Parameters + "]");
            Debug.WriteLine("Virtual Path: " + ftpServer.VirtualPath);
            Debug.WriteLine("Local Path:   " + ftpServer.LocalPath);

            if (e.Command == "SITE")
            {
                if (e.Parameters.ToUpper() == "TIME")
                {
                    ftpServer.SendResponse(e.ClientId, 200, "The local time is " + DateTime.Now.ToString("G"));
                }
            }
        }

        private void ftpServer_OnConnect(System.Object eventSender, SocketTools.FtpServer.ConnectEventArgs e)
        {
            Debug.WriteLine("Client " + e.ClientId + " connected from " + e.ClientAddress);
            UpdateStatus("Client " + e.ClientId + " connected from " + e.ClientAddress);
        }

        private void ftpServer_OnDisconnect(System.Object eventSender, SocketTools.FtpServer.DisconnectEventArgs e)
        {
            Debug.WriteLine("Client " + e.ClientId + " disconnected");
            UpdateStatus("Client " + e.ClientId + " disconnected from the server");
        }

        private void ftpServer_OnDownload(System.Object eventSender, SocketTools.FtpServer.DownloadEventArgs e)
        {
            Debug.WriteLine("Client " + e.ClientId + " downloaded file " + e.FileName);
        }

        private void ftpServer_OnError(System.Object eventSender, SocketTools.FtpServer.ErrorEventArgs e)
        {
            Debug.WriteLine("Client " + e.ClientId + " error " + e.Error + ": " + e.Description);
        }

        private void ftpServer_OnExecute(System.Object eventSender, SocketTools.FtpServer.ExecuteEventArgs e)
        {
            Debug.WriteLine("Client " + e.ClientId + " executed " + e.Program + " (" + e.ExitCode + ")");
        }

        private void ftpServer_OnIdle(System.Object eventSender, System.EventArgs e)
        {
            Debug.WriteLine("All clients have disconnected, the server is idle");
        }

        private void ftpServer_OnLogin(System.Object eventSender, SocketTools.FtpServer.LoginEventArgs e)
        {
            Debug.WriteLine("Client " + e.ClientId + " logged in as " + e.UserName + " (" + e.Directory + ")");
        }

        private void ftpServer_OnLogout(System.Object eventSender, SocketTools.FtpServer.LogoutEventArgs e)
        {
            Debug.WriteLine("Client " + e.ClientId + " logged out as " + e.UserName);
        }

        private void ftpServer_OnResult(System.Object eventSender, SocketTools.FtpServer.ResultEventArgs e)
        {
            Debug.WriteLine("Client " + e.ClientId + " command " + e.Command + " completed (" + e.ResultCode + ")");
        }

        private void ftpServer_OnStart(System.Object eventSender, System.EventArgs e)
        {
            Debug.WriteLine("Server has started accepting connections");
            UpdateStatus("Server accepting connections on " + ftpServer.ServerAddress + ":" + ftpServer.ServerPort);
            UpdateForm(true);
        }

        private void ftpServer_OnStop(System.Object eventSender, System.EventArgs e)
        {
            Debug.WriteLine("Server has stopped, all clients terminated");
            UpdateStatus("Server stopped accepting connections");
            UpdateForm(true);
        }

        private void ftpServer_OnTimeout(System.Object eventSender, SocketTools.FtpServer.TimeoutEventArgs e)
        {
            Debug.WriteLine("Client " + e.ClientId + " timed out after " + e.Elapsed + " seconds");
        }

        private void ftpServer_OnUpload(System.Object eventSender, SocketTools.FtpServer.UploadEventArgs e)
        {
            Debug.WriteLine("Client " + e.ClientId + " uploaded file " + e.FileName);
        }
    }
} 
